#ifndef __FTFont_bi__
#define __FTFont_bi__

type makeglyphCallback as function cdecl (slot as FT_GlyphSlot, data_ as any ptr) as FTGLglyph  ptr

/'*
 * FTGLfont is the public interface for the FTGL library.
 *
 * It is good practice after using these functions to test the error
 * code returned. <code>FT_Error Error()</code>. Check the freetype file
 * fterrdef.h for error definitions.
 '/
type _FTGLFont as any :type FTGLfont as _FTGLfont

extern "c"

/'*
 * Create a custom FTGL font object.
 *
 * @param fontFilePath  The font file name.
 * @param data  A pointer to private data that will be passed to callbacks.
 * @param makeglyphCallback  A glyph-making callback function.
 * @return  An FTGLfont* object.
 '/
declare function ftglCreateCustomFont(fontFilePath as const zstring ptr, data_ as any ptr, mgcb as makeglyphCallback) as FTGLfont ptr

/'*
 * Destroy an FTGL font object.
 *
 * @param font  An FTGLfont* object.
 '/
declare sub ftglDestroyFont(font as FTGLfont ptr)

/'*
 * Attach auxilliary file to font e.g. font metrics.
 *
 * Note: not all font formats implement this function.
 *
 * @param font  An FTGLfont* object.
 * @param path  Auxilliary font file path.
 * @return  1 if file has been attached successfully.
 '/
declare function ftglAttachFile(font as FTGLfont ptr, path as const zstring ptr) as integer

/'*
 * Attach auxilliary data to font, e.g. font metrics, from memory.
 *
 * Note: not all font formats implement this function.
 *
 * @param font  An FTGLfont* object.
 * @param data  The in-memory buffer.
 * @param size  The length of the buffer in bytes.
 * @return  1 if file has been attached successfully.
 '/
declare function ftglAttachData(font as FTGLfont ptr, data_ as const ubyte ptr, size as uinteger) as integer

/'*
 * Set the character map for the face.
 *
 * @param font  An FTGLfont* object.
 * @param encoding  Freetype enumerate for char map code.
 * @return  1 if charmap was valid and set correctly.
 '/
declare function ftglSetFontCharMap(font as FTGLfont ptr, encoding_ as FT_Encoding) as integer

/'*
 * Get the number of character maps in this face.
 *
 * @param font  An FTGLfont* object.
 * @return character map count.
 '/
declare function ftglGetFontCharMapCount(font as FTGLfont ptr) as uinteger

/'*
 * Get a list of character maps in this face.
 *
 * @param font  An FTGLfont* object.
 * @return pointer to the first encoding.
 '/
declare function ftglGetFontCharMapList(font as FTGLfont ptr) as FT_Encoding ptr

/'*
 * Set the char size for the current face.
 *
 * @param font  An FTGLfont* object.
 * @param size  The face size in points (1/72 inch).
 * @param res  The resolution of the target device, or 0 to use the default
 *             value of 72.
 * @return  1 if size was set correctly.
 '/
declare function ftglSetFontFaceSize(font as FTGLfont ptr, size as uinteger, res as uinteger=0) as integer

/'*
 * Get the current face size in points (1/72 inch).
 *
 * @param font  An FTGLfont* object.
 * @return face size
 '/
declare function ftglGetFontFaceSize(font as FTGLfont ptr) as uinteger

/'*
 * Set the extrusion distance for the font. Only implemented by FTExtrudeFont.
 *
 * @param font  An FTGLfont* object.
 * @param depth  The extrusion distance.
 '/
declare sub ftglSetFontDepth(font as FTGLfont ptr, depth as single)

/'*
 * Set the outset distance for the font. Only FTOutlineFont, FTPolygonFont and 
 * FTExtrudeFont implement front outset. Only FTExtrudeFont implements back outset.
 *
 * @param font  An FTGLfont* object.
 * @param front  The front outset distance.
 * @param back  The back outset distance.
 '/
declare sub ftglSetFontOutset(font as FTGLfont ptr, front as single, back as single)

/'*
 * Enable or disable the use of Display Lists inside FTGL.
 *
 * @param font  An FTGLfont* object.
 * @param useList  1 turns ON display lists.
 *                 0 turns OFF display lists.
 '/
declare sub ftglSetFontDisplayList(font as FTGLfont ptr, useList as integer)

/'*
 * Get the global ascender height for the face.
 *
 * @param font  An FTGLfont* object.
 * @return  Ascender height
 '/
declare function ftglGetFontAscender(font as FTGLfont ptr) as single

/'*
 * Gets the global descender height for the face.
 *
 * @param font  An FTGLfont* object.
 * @return  Descender height
 '/
declare function ftglGetFontDescender(font as FTGLfont ptr) as single

/'*
 * Gets the line spacing for the font.
 *
 * @param font  An FTGLfont* object.
 * @return  Line height
 '/
declare function ftglGetFontLineHeight(font as FTGLfont ptr) as single

/'*
 * Get the bounding box for a string.
 *
 * @param font  An FTGLfont* object.
 * @param string  A char buffer
 * @param len  The length of the string. If < 0 then all characters will be checked until a null character is encountered (optional).
 * @param bounds  An array of 6 float values where the bounding box's lower left near and upper right far 3D coordinates will be stored.
 '/
declare sub ftglGetFontBBox(font as FTGLfont ptr, string_ as const zstring ptr,len_ as integer, bounds as single ptr)

/'*
 * Get the advance width for a string.
 *
 * @param font  An FTGLfont* object.
 * @param string  A char string.
 * @return  Advance width
 '/
declare function ftglGetFontAdvance(font as FTGLfont ptr, string_ as const zstring ptr) as single

/'*
 * Render a string of characters.
 *
 * @param font  An FTGLfont* object.
 * @param string  Char string to be output.
 * @param mode  Render mode to display.
 '/
declare sub ftglRenderFont(font as FTGLfont ptr, string_ as const zstring ptr, mode as integer)

/'*
 * Query a font for errors.
 *
 * @param font  An FTGLfont* object.
 * @return  The current error code.
 '/
declare function ftglGetFontError(font as FTGLfont ptr) as FT_Error

end extern

#endif
