#ifndef __IM_CONVERT_BI__
#define __IM_CONVERT_BI__

#include once "im_image.bi"

' Image Conversion
' Converts one type of image into another. Can convert between color modes and between data types.

' Complex to real conversions
enum imComplex2Real 
  IM_CPX_REAL, 
  IM_CPX_IMAG, 
  IM_CPX_MAG, 
  IM_CPX_PHASE
end enum

' Predefined Gamma factors. Gamma can be any real number.
' When gamma<0 use logarithmic, when gamma>0 use exponential.
' gamma(x,g) = ((e^(g*x))-1)/(exp(g)-1)
' gamma(x,g) = (log((g*x)+1))/(log(g+1))
enum imGammaFactor
  IM_GAMMA_LINEAR   = 0,
  IM_GAMMA_LOGLITE  = -10,
  IM_GAMMA_LOGHEAVY = -1000,
  IM_GAMMA_EXPLITE  = 2,
  IM_GAMMA_EXPHEAVY = 7
end enum

' Predefined Cast Modes
enum imCastMode
  IM_CAST_MINMAX, ' scan for min and max values
  IM_CAST_FIXED,  ' use predefied 0-max values, see \ref color Color Manipulation.
  IM_CAST_DIRECT  ' direct type cast the value. Only byte and ushort will be cropped.
end enum

extern "c"

' Changes the image data type, using a complex2real conversion, a gamma factor, and an abssolute mode (modulus).
' When demoting the data type the function will scan for min/max values or use fixed values (cast_mode)
' to scale the result according to the destiny range.
' Except complex to real that will use only the complex2real conversion.
' Images must be of the same size and color mode.
' Returns IM_ERR_NONE, IM_ERR_DATA or IM_ERR_COUNTER
_DF_ imConvertDataType(src_img as const imImage ptr, dst_img as imImage ptr, cpx2real as integer, gamma as single, abssolute as integer, cast_mode as integer) as integer

' Converts one color space to another. Images must be of the same size and data type.
' CMYK can be converted to RGB only, and it is a very simple conversion.
' All colors can be converted to Binary, the non zero gray values are converted to 1.
' RGB to Map uses the median cut implementation from the free IJG JPEG software, copyright Thomas G. Lane.
' Alpha channel is considered and Transparency* attributes are converted to alpha channel.
' All other color space conversions assume sRGB and CIE definitions.
' Returns IM_ERR_NONE, IM_ERR_DATA or IM_ERR_COUNTER.
_DF_ imConvertColorSpace(src_img as const imImage ptr, dst_img as imImage ptr) as integer

' Converts the image to its bitmap equivalent, uses imConvertColorSpace and imConvertDataType.
' Returns IM_ERR_NONE, IM_ERR_DATA or IM_ERR_COUNTER.
' The function im.ConvertToBitmapNew uses the default convertion result from \ref imColorModeToBitmap if color_space is nil.
_DF_ imConvertToBitmap(src_img as const imImage ptr, dst_img as imImage ptr, cpx2real as integer, gamma as single, abssolute as integer, cast_mode as integer) as integer

' Returns an OpenGL compatible data buffer. Also returns the correspondant pixel format.
' The memory allocated is stored in the attribute "GLDATA" with BYTE type. And it will exists while the image exists.
' It can be cleared by setting the attribute to NULL.
' MAP images are converted to RGB, and BINARY images are converted to GRAY.
' Alpha channel is considered and Transparency* attributes are converted to alpha channel.
' So calculate depth from glformat, not from image depth.
_DF_ imImageGetOpenGLData(img as const imImage ptr, glformat as integer ptr) as any ptr

' Raw Data Conversion Utilities
' Changes the packing of the data buffer. Both must have the same depth.
_DS_ imConvertPacking(src_data as const any ptr, dst_data as any ptr, w as integer, h as integer, depth as integer, data_type as integer, src_is_packed as integer)

' Changes in-place a MAP data into a RGB data. The data must have room for the RGB image.
' depth can be 3 or 4. count=width*height.
_DS_ imConvertMapToRGB(data_ as ubyte ptr, count as integer, depth as integer, packed as integer, palette_ as long ptr, palette_count as integer)

end extern


#endif
