#ifndef __IM_PROC_ANA_BI__
#define __IM_PROC_ANA_BI__

' see copyright notice at the file iup.txt

#include once "im_image.bi"


' Numerical Statistics Structure
type _imStats
  as single max       ' Maximum value
  as single min       ' Minimum value
  as ulong positive   ' Number of Positive Values
  as ulong negative   ' Number of Negative Values
  as ulong zeros      ' Number of Zeros
  as single mean      ' Mean
  as single stddev    ' Standard Deviation
end type
type imStats as _imStats

extern "c"


' Image Statistics Calculations
' Operations to calculate some statistics over images.

' Calculates the RMS error between two images (Root Mean Square Error).
_DF_ imCalcRMSError(img1 as const imImage ptr, img2 as const imImage ptr) as single

' Calculates the SNR of an image and its noise (Signal Noise Ratio).
_DF_ imCalcSNR(src_image as const imImage ptr, noise_image as const imImage ptr) as single

' Count the number of different colors in an image. 
_DF_ imCalcCountColors(img as const imImage ptr) as ulong

' Calculates the histogram of a IM_BYTE data. 
' Histogram is always 256 positions long. 
' When cumulative is different from zero it calculates the cumulative histogram.
_DS_ imCalcHistogram(data_ as const ubyte ptr, count as integer, histo as ulong ptr, cumulative as integer)

' Calculates the histogram of a IM_USHORT data. 
' Histogram is always 65535 positions long. 
' When cumulative is different from zero it calculates the cumulative histogram. 
_DS_ imCalcUShortHistogram(data_ as const ushort ptr, count as integer, histo as ulong ptr, cumulative as integer)

' Calculates the gray histogram of an image. 
' Image must be IM_BYTE/(IM_RGB, IM_GRAY, IM_BINARY or IM_MAP). 
' If the image is IM_RGB then the histogram of the luma component is calculated. 
' Histogram is always 256 positions long. 
' When cumulative is different from zero it calculates the cumulative histogram.
_DS_ imCalcGrayHistogram(img as const imImage ptr, histo as ulong ptr, cumulative as integer)


' Calculates the statistics about the image data. 
' There is one stats for each depth plane. For ex: stats[0]=red stats, stats[0]=green stats, ... 
' Supports all data types except IM_CFLOAT. 
_DS_ imCalcImageStatistics(img as const imImage ptr, stats as imStats ptr)

' Calculates the statistics about the image histogram data.
' There is one stats for each depth plane. For ex: stats[0]=red stats, stats[0]=green stats, ... 
' Only IM_BYTE images are supported.
_DS_ imCalcHistogramStatistics(img as const imImage ptr, stats as imStats ptr)

' Calculates some extra statistics about the image histogram data.
' There is one stats for each depth plane. 
' Only IM_BYTE images are supported. 
' mode will be -1 if more than one max is found.
_DS_ imCalcHistoImageStatistics(img as const imImage ptr, median as integer ptr, mode as integer ptr)


' Image Analysis

' Find white regions in binary image. 
' Result is IM_GRAY/IM_USHORT type. Regions can be 4 connected or 8 connected. 
' Returns the number of regions found. Background is marked as 0. 
' Regions touching the border are considered only if touch_border=1.
_DF_ imAnalyzeFindRegions(src_image as const imImage ptr, dst_image as imImage ptr, connect as integer, touch_border as integer) as integer

' Measure the actual area of all regions. Holes are not included. 
' This is the number of pixels of each region. 
' Source image is IM_GRAY/IM_USHORT type (the result of \ref imAnalyzeFindRegions). 
' area has size the number of regions.
_DS_ imAnalyzeMeasureArea(img as const imImage ptr, area as integer ptr, region_count as integer)

' Measure the polygonal area limited by the perimeter line of all regions. Holes are not included. 
' Notice that some regions may have polygonal area zero. 
' Source image is IM_GRAY/IM_USHORT type (the result of \ref imAnalyzeFindRegions). 
' perimarea has size the number of regions.
_DS_ imAnalyzeMeasurePerimArea(img as const imImage ptr, perimarea as single ptr)

' Calculate the centroid position of all regions. Holes are not included. 
' Source image is IM_GRAY/IM_USHORT type (the result of \ref imAnalyzeFindRegions). 
' area, cx and cy have size the number of regions. If area is NULL will be internally calculated.
_DS_ imAnalyzeMeasureCentroid(img as const imImage ptr, area as const integer ptr, region_count as integer, cx as single ptr, cy as single ptr)

' Calculate the principal major axis slope of all regions. 
' Source image is IM_GRAY/IM_USHORT type (the result of \ref imAnalyzeFindRegions). 
' data has size the number of regions. If area or centroid are NULL will be internally calculated. 
' Principal (major and minor) axes are defined to be those axes that pass through the
' centroid, about which the moment of inertia of the region is, respectively maximal or minimal.
_DS_ imAnalyzeMeasurePrincipalAxis(img as const imImage ptr, area as const integer ptr, cx as const single ptr, cy as const single ptr, _ 
                                   region_count as const integer, major_slope as single ptr, major_length as single ptr, _
                                   minor_slope as single ptr, minor_length as single ptr)

' Measure the number and area of holes of all regions. 
' Source image is IM_USHORT type (the result of \ref imAnalyzeFindRegions). 
' area and perim has size the number of regions, if some is NULL it will be not calculated.
_DS_ imAnalyzeMeasureHoles(img as const imImage ptr, connect as integer, holes_count as integer ptr, area as integer ptr, perim as single ptr)

' Measure the total perimeter of all regions (external and internal). 
' Source image is IM_GRAY/IM_USHORT type (the result of imAnalyzeFindRegions). 
' It uses a half-pixel inter distance for 8 neighboors in a perimeter of a 4 connected region. 
' This function can also be used to measure line lenght. 
' perim has size the number of regions.
_DS_ imAnalyzeMeasurePerimeter(img as const imImage ptr, perim as single ptr, region_count as integer)

'Isolates the perimeter line of gray integer images. Background is defined as being black (0). 
' It just checks if at least one of the 4 connected neighboors is non zero. Image borders are extended with zeros.
_DS_ imProcessPerimeterLine(src_image as const imImage ptr, dst_image as imImage ptr)

' Eliminates regions that have area size outside or inside the given interval. 
' Source and destiny are a binary images. Regions can be 4 connected or 8 connected. 
' Can be done in-place. end_size can be zero to indicate no upper limit or an area with width*height size. 
_DS_ imProcessRemoveByArea(src_image as const imImage ptr, dst_image as imImage ptr, connect as integer, start_size as integer, end_size as integer, inside as integer)

' Fill holes inside white regions. 
' Source and destiny are a binary images. Regions can be 4 connected or 8 connected. 
' Can be done in-place. 
_DS_ imProcessFillHoles(src_image as const imImage ptr, dst_image as imImage ptr, connect as integer)

end extern

#endif ' __IM_PROC_ANA_BI__
