#ifndef __IM_PROCESS_GLO_BI__
#define __IM_PROCESS_GLO_BI__

' see copyright notice at the file iup.txt

#include once "im_image.bi"


extern "c"

' Other Domain Transform Operations

' Hough Lines Transform.
' It will detect white lines in a black background.
' So the source image must be a IM_BINARY image with the white lines of interest enhanced. 
' The better the threshold with the white lines the better the line detection.
' The destiny image must have IM_GRAY, IM_INT, hg_width=180, hg_height=2*rmax+1, 
' where rmax is the image diagonal/2 (rmax = srqrt(width*width + height*height)).
' The hough transform defines  "cos(theta) * X  + sin(theta) * Y = rho" and the parameters are in the interval:
' theta = "0 .. 179", rho = "-hg_height/2 .. hg_height/2" .
' Where rho is the perpendicular distance from the center of the image and theta the angle with the normal.
' So do not confuse theta with the line angle, they are perpendicular.
' Returns zero if the counter aborted.
_DF_ imProcessHoughLines(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Draw detected hough lines.
' The source image must be IM_GRAY and IM_BYTE.
' The destiny image can be a clone of the source image or it can be the source image for in place processing.
' If the hough transform is not NULL, then the hough points are filtered to include only lines
' that are significally different from each other.
' The hough image is the hough transform image, but it is optional and can be NULL. 
' If not NULL then it will be used to filter lines that are very similar.
' The hough points image is a hough transform image that was thresholded to a IM_BINARY image, 
' usually using a Local Max threshold operation (see imProcessLocalMaxThreshold). 
' Again the better the threshold the better the results.
' The destiny image will be set to IM_MAP, and the detected lines will be drawn using a red color.
' Returns the number of detected lines.
_DF_ imProcessHoughLinesDraw(src_image as const imImage ptr, hough as const imImage ptr, hough_points as const imImage ptr, dst_image as imImage ptr) as integer

' Calculates the Cross Correlation in the frequency domain. 
' CrossCorr(a,b) = IFFT(Conj(FFT(a))*FFT(b))
' Images must be of the same size and only destiny image must be of type complex.
_DS_ imProcessCrossCorrelation(src_image1 as const imImage ptr, src_image2 as const imImage ptr, dst_image as imImage ptr)

' Calculates the Auto Correlation in the frequency domain. 
' Uses the cross correlation.
' Images must be of the same size and only destiny image must be of type complex.
_DS_ imProcessAutoCorrelation(src_image as const imImage ptr, dst_image as imImage ptr)

' Calculates the Distance Transform of a binary image using an aproximation of the euclidian distance.
' Each white pixel in the binary image is assigned a value equal to its distance from the nearest black pixel.
' Uses a two-pass algorithm incrementally calculating the distance.
' Source image must be IM_BINARY, destiny must be IM_FLOAT.
_DS_ imProcessDistanceTransform(src_image as const imImage ptr, dst_image as imImage ptr)

' Marks all the regional maximum of the distance transform.
' source is IMGRAY/IM_FLOAT destiny in IM_BINARY.
' We consider maximum all connected pixel values that have smaller pixel values around it.
_DS_ imProcessRegionalMaximum(src_image as const imImage ptr, dst_image as imImage ptr)

' Fourier Transform Operations

' All Fourier transforms use FFTW library.
' The pre-compiled binaries for FFTW version 2.1.5 includes all the necessary files.
' The pre-compiled binaries for FFTW version 3.x depends on an external library, not provided.
' To build the code that uses FFTW version 3 you must define USE_FFTW3.

' IMPORTANT: The FFTW lib has a GPL license. The license of the "im_fftw" library is automatically the GPL.
' So you cannot use it for commercial applications without contacting the authors. 

' Forward FFT.
' The result has its lowest frequency at the center of the image. This is an unnormalized fft.
' Images must be of the same size. Destiny image must be of type complex.
_DS_ imProcessFFT(src_image as const imImage ptr, dst_image as imImage ptr)

' Inverse FFT.
' The image has its lowest frequency restored to the origin before the transform.
' The result is normalized by (width*height).
' Images must be of the same size and both must be of type complex.
_DS_ imProcessIFFT(src_image as const imImage ptr, dst_image as imImage ptr)

' Raw in-place FFT (forward or inverse).
' The lowest frequency can be centered after forward, or can be restored to the origin before inverse.
' The result can be normalized after the transform by sqrt(w*h) [1] or by (w*h) [2], or left unnormalized [0].
' Images must be of the same size and both must be of type complex.
_DS_ imProcessFFTraw(image as imImage ptr, inverse as integer, center as integer, normalize as integer)

' Auxiliary function for the raw FFT. 
' This is the function used internally to change the lowest frequency position in the image.
' If the image size has even dimensions the flag "center2origin" is useless.
' But if it is odd, you must specify if its from center to origin (usually used before inverse)
' or from origin to center (usually used after forward).
' Notice that this function is used for images in the the frequency domain.
' Image type must be complex.
_DS_ imProcessSwapQuadrants(image as imImage ptr, center2origin as integer)

end extern

#endif ' __IM_PROCESS_GLO_BI__
