#ifndef __IUP_CLASS_OPT_BI__
#define __IUP_CLASS_OPT_BI__

' see copyright notice at the file iup.txt

' optional

#include once "iup_table_opt.bi"

' Known native types.
enum InativeType
  IUP_TYPEVOID,     ' No native representation - HBOX, VBOX, ZBOX, FILL, RADIO (handle=&HFFFFFFFF -1 always)
  IUP_TYPECONTROL,  ' Native controls - BUTTON, LABEL, TOGGLE, LIST, TEXT, MULTILINE, FRAME, others
  IUP_TYPECANVAS,   ' Drawing canvas, also used as a base control for custom controls. 
  IUP_TYPEDIALOG,   ' DIALOG
  IUP_TYPEIMAGE,    ' IMAGE
  IUP_TYPEMENU      ' MENU, SUBMENU, ITEM, SEPARATOR
end enum

' Possible number of children.
enum IchildType
  IUP_CHILDNONE, ' can not add children using Append/Insert
  IUP_CHILDMANY  ' can add any number of children IUP_CHILDMANY+n can add n children.
end enum

' Attribute flags.
' Used by iupClassRegisterAttribute.
enum IattribFlags
  IUPAF_DEFAULT         =    0, ' inheritable, can has a default value, is a string, 
                                ' can call the set/get functions only if mapped, no ID
  IUPAF_NO_INHERIT      =    1, ' is not inheritable
  IUPAF_NO_DEFAULTVALUE =    2, ' has no default value
  IUPAF_NO_STRING       =    4, ' is not a string
  IUPAF_NOT_MAPPED      =    8, ' will call the set/get functions also when not mapped
  IUPAF_HAS_ID          =   16, ' can has an ID at the end of the name, automatically set by iupClassRegisterAttributeId */
  IUPAF_READONLY        =   32, ' is read-only, can not be changed
  IUPAF_WRITEONLY       =   64, ' is write-only, usually an action
  IUPAF_HAS_ID2         =  128, ' can has two IDs at the end of the name, automatically set by iupClassRegisterAttributeId2
  IUPAF_CALLBACK        =  256, ' is a callback, not an attribute
  IUPAF_NO_SAVE         =  512, ' can NOT be directly saved, should have at least manual processing
  IUPAF_NOT_SUPPORTED   = 1024, ' not supported in that driver
  IUPAF_IHANDLENAME     = 2048  ' is an Ihandle* name, associated with IupSetHandle
end enum

type IClass as Iclass_

type Iclass_
  ' configuration parameters.
  as zstring ptr name     ' class name. No default, must be initialized.
  as zstring ptr format   ' Creation parameters format of the class.
                          ' Used only for LED parsing.
                          ' It can have none (NULL), one or more of the following.
                          ' "b" = (unsigned char) - byte
                          ' "c" = (unsigned char*) - array of byte
                          ' "i" = (int) - integer
                          ' "j" = (int*) - array of integer
                          ' "f" = (float) - real
                          ' "s" = (char*) - string 
                          ' "a" = (char*) - name of the ACTION callback
                          ' "h" = (Ihandle*) - element handle
                          ' "g" = (Ihandle**) - array of element handle
                   
  as InativeType nativetype ' native type. Default is IUP_TYPEVOID.
  as IchildType  childtype  ' children count enum: none, many, or n, as described in \ref IchildType. Default is IUP_CHILDNONE.
                            ' This identifies a container that can be manipulated with IupReparent, IupAppend and IupInsert.
                            ' Used to control the allowed number of children and define its behavior in the layout processing.
                            ' The element can still have hidden children even if this is none.
  as integer is_interactive ' keyboard interactive boolean, 
                            ' true if the class can have the keyboard input focus. Default is false.
  as integer has_attrib_id  ' indicate if any attribute is numbered. Default is not. Can be 1 or 2.

  as Iclass_ ptr parent     ' class parent to implement inheritance.
                            ' Class name must be different.
                            ' Creation parameters should be the same or repace the parents creation function.
                            ' Native type should be the same.
                            ' Child type should be a more restrictive or equal type (many->one->none).
                            ' Attribute functions will have only one common table.
                            ' All methods can be changed, set to NULL, switched, etc.

  as Itable ptr attrib_func ' table of functions to handle attributes, only one per class tree

  ' Class methods.

  ' Method that allocates a new instance of the class.
  ' Used by inherited classes in \ref iupClassNew.
  new_ as function cdecl () as Iclass ptr

  ' Method that release the memory allocated by the class.
  ' Called only once at \ref iupClassRelease.
  release as sub cdecl (ic as Iclass ptr)

  ' Method that creates the element and process the creation parameters.
  ' Called only from IupCreate.
  ' The parameters can be NULL for all the controls.
  ' The control should also depend on attributes set before IupMap.
  ' Must return IUP_NOERROR or IUP_ERROR.
  ' Can be NULL, like all methods.
  create as function cdecl (ih as Ihandle ptr, params as any ptr ptr) as integer

  ' Method that map (create) the control to the native system.
  ' Called only from IupMap.
  ' Must return IUP_NOERROR or IUP_ERROR.
  map as function cdecl (ih as Ihandle ptr) as integer

  ' Method that unmap (destroy) the control from the native system.
  ' Called only from IupUnmap if the control is mapped.
  unmap as sub cdecl (ih as ihandle ptr)

  ' Method that destroys the element. \n
  ' Called only from IupDestroy. Always called even if the control is not mapped.
  destroy as sub cdecl (ih as Ihandle ptr)

  ' Returns the actual parent to add a child. The default implementation returns itself.
  ' Called only from IupAppend or IupReparent.
  ' This allows IUP elements to be a combination of other IUP elements in a single IUP element.
  GetInnerContainer as function cdecl (ih as Ihandle ptr) as Ihandle ptr

  ' Returns the internal native parent. The default implementation returns the handle of itself
  ' Called from \ref iupChildTreeGetNativeParentHandle.
  ' This allows native elements to have an internal container
  ' that will be the actual native parent, or in other words allows native elements to be a combination of 
  ' other native elements in a single IUP element. 
  ' The actual native parent may depend on the child tree (see IupTabs for an example).
  GetInnerNativeContainerHandle as function cdecl (ih as Ihandle ptr, child as Ihandle ptr) as any ptr

  ' Notifies the element that a child was appended using IupAppend.
  ' Called only from IupAppend or IupReparent. The child is not mapped, but the parent can be mapped.
  ChildAdded as sub cdecl (ih as Ihandle ptr, child as Ihandle ptr)

  ' Notifies the element that a child was removed using IupDetach.
  ' Called only from IupDetach or IupReparent. The child is not mapped, but the parent can be mapped.
  ChildRemoved as sub cdecl (ih as Ihandle ptr, child as Ihandle ptr)

  ' Method that update size and position of the native control.
  ' Called only from iupLayoutUpdate and if the element is mapped.
  LayoutUpdate as sub cdecl (ih as Ihandle ptr)

  ' Method that computes the natural size based on the user size and the actual natural size.
  ' Should update expand if a container, but does NOT depends on expand to compute the natural size.
  ' Must call the iupBaseComputeNaturalSize for each children.
  ' First calculate the native size for the children, then for the element.
  ' Also called before the element is mapped, so it must be independent of the native control.
  ' First call done at iupLayoutCompute for the dialog.
  ComputeNaturalSize as sub cdecl (ih as Ihandle ptr, w as integer ptr, h as integer ptr, children_expand as integer ptr)

  ' Method that calculates and updates the current size of children based on the available size,
  ' the natural size and the expand configuration.
  ' Called only if there is any children.
  ' Must call \ref iupBaseSetCurrentSize for each children. 
  ' shrink is the dialog attribute passed here for optimization.
  ' Also called before the element is mapped, so it must be independent of the native control.
  ' First call done at iupLayoutCompute for the dialog.
  SetChildrenCurrentSize as sub cdecl (ih as Ihandle ptr, shrink as integer)

  ' Method that calculates and updates the position relative to the parent.
  ' Called only if there is any children.
  ' Must call iupBaseSetPosition for each children.
  ' Also called before the element is mapped, so it must be independent of the native control.
  ' First call done at iupLayoutCompute for the dialog.
  SetChildrenPosition as sub cdecl (ih as Ihandle ptr, x as integer, y as integer)

  ' Method that shows a popup dialog. Called only for native pre-defined dialogs.
  ' The element is not mapped.
  ' Must return IUP_ERROR or IUP_NOERROR.
  ' Called only from iupDialogPopup.
   DlgPopup as function cdecl (ih as Ihandle ptr, x as integer, y as integer) as integer
end type


' GetAttribute called for a specific attribute.
' Used by iupClassRegisterAttribute.
type IattribGetFunc as function cdecl (ih as Ihandle ptr) as zstring ptr

' GetAttribute called for a specific attribute when has_attrib_id is 1.
' Same as IattribGetFunc but handle attribute names with number ids at the end.
' When calling iupClassRegisterAttribute just use a typecast.
' -1 is used for invalid ids.
' Pure numbers are translated into IDVALUEid. Used by iupClassRegisterAttribute.
type IattribGetIdFunc as function cdecl (ih as Ihandle ptr, id as integer) as zstring ptr

' GetAttribute called for a specific attribute when has_attrib_id is 1.
' Same as IattribGetFunc but handle attribute names with number ids at the end.
' When calling iupClassRegisterAttribute just use a typecast.
' -1 is used for invalid ids.
' Pure numbers are translated into IDVALUEid. Used by iupClassRegisterAttribute.
type IattribGetId2Func as function cdecl (ih as Ihandle ptr, id1 as integer, id2 as integer) as zstring ptr

' SetAttribute called for a specific attribute.
' If returns 0, the attribute will not be stored in the hash table
' (except inheritble attributes that are always stored in the hash table).
' When IupSetAttribute is called using value=NULL, the default_value is passed to this function.
' Used by \ref iupClassRegisterAttribute.
type IattribSetFunc as function cdecl (ih as Ihandle ptr, value as const zstring ptr) as integer

' SetAttribute called for a specific attribute when has_attrib_id is 1.
' Same as IattribSetFunc but handle attribute names with number ids at the end.
' When calling iupClassRegisterAttribute just use a typecast.
' -1 is used for invalid ids.
' Pure numbers are translated into IDVALUEid, ex: "1" = "IDVALUE1".
' Used by \ref iupClassRegisterAttribute.
type IattribSetIdFunc as function cdecl (ih as Ihandle ptr, id as integer ,value as const zstring ptr) as integer

' SetAttribute called for a specific attribute when has_attrib_id is 2.
' Same as IattribSetFunc but handle attribute names with number ids at the end.
' When calling iupClassRegisterAttribute just use a typecast.
' -1 is used for invalid ids.
' Pure numbers are translated into IDVALUEid, ex: "1" = "IDVALUE1".
' Used by \ref iupClassRegisterAttribute.
type IattribSetId2Func as function cdecl (ih as Ihandle ptr, id1 as integer, id2 as integer, value as const zstring ptr) as integer

' means that the default value is the same as the system default value, 
' used only in \ref iupClassRegisterAttribute
#define IUPAF_SAMEASSYSTEM cptr(as zstrng ptr,-1)

extern "c"

' Allocates memory for the Iclass structure and initializes the attribute handling functions table.
' If parent is spcified then a new instance of the parent class is created and set as the actual parent class.
_DF_ iupClassNew( ic_parent as Iclass ptr) as Iclass ptr

' Release the memory allocated by the class.
' Calls the \ref Iclass::Release method. Called from iupRegisterFinish.
_DS_ iupClassRelease(ic as Iclass ptr)

' Check if the class name match the given name.
' Parent classes are also checked.
_DF_ iupClassMatchopt alias "iupClassMatch"(ic as Iclass ptr, classname as const zstring ptr) as integer


' Register attribute handling functions, defaults and flags. get, set and default_value can be NULL.
' default_value should point to a constant string, it will not be duplicated internally.
' Notice that when an attribute is not defined then default_value=NULL, 
' is inheritable can has a default value and is a string.
' Since there is only one attribute function table per class tree, 
' if you register the same attribute in a child class, then it will replace the parent registration.
' If an attribute is not inheritable or not a string then it MUST be registered.
' Internal attributes (starting with "_IUP") can never be registered.
_DS_ iupClassRegisterAttribute(ic as Iclass ptr, name_ as const zstring ptr, _
                                           get_ as IattribGetFunc, _
                                           set_ as IattribSetFunc, _
                                           default_value as const zstring ptr, _
                                           system_default as const zstring ptr, _
                                           flags as integer)

' Same as iupClassRegisterAttribute for attributes with Ids.
_DS_ iupClassRegisterAttributeId(ic as Iclass ptr, name_ as const zstring ptr, _
                                           get_ as IattribGetFunc, _
                                           set_ as IattribSetFunc, _
                                           flags as integer)

' Same as iupClassRegisterAttribute for attributes with two Ids.
_DS_ iupClassRegisterAttributeId2(ic as Iclass ptr, name_ as const zstring ptr, _
                                            get_ as IattribGetId2Func, _
                                            set_ as IattribSetId2Func, _
                                           flags as integer)

' Returns the attribute handling functions, defaults and flags.
_DS_ iupClassRegisterGetAttribute(ic as Iclass ptr, name_ as const zstring ptr, _
                                           get_ as IattribGetFunc ptr, _
                                           set_ as IattribSetFunc ptr, _
                                           default_value  as const zstring ptr ptr, _
                                           system_default as const zstring ptr ptr, _
                                           flags as integer ptr)

' Replaces the attribute handling functions of an already registered attribute.
_DS_ iupClassRegisterReplaceAttribFunc(ic as Iclass ptr, name_ as const zstring ptr,  get_ as IattribGetFunc,  set_ as IattribSetFunc)

' Replaces the attribute handling default of an already registered attribute.
_DS_ iupClassRegisterReplaceAttribDef(ic as Iclass ptr, name_ as const zstring ptr, _default_value as const zstring ptr, _system_default as const zstring ptr)

' Replaces the attribute handling functions of an already registered attribute.
_DS_ iupClassRegisterReplaceAttribFlags(ic as Iclass ptr, name_ as const zstring ptr, flags as integer)

' Register the parameters of a callback.
' format follows the format specification of the class creation parameters format, 
' but it adds the "double" option and remove array options.
' It can have none, one or more of the following.
' - "b" = (unsigned char) - byte
' - "i" = (int) - integer
' - "f" = (float) - real
' - "d" = (double) - real
' - "s" = (char*) - string 
' - "v" = (void*) - generic pointer 
' - "h" = (Ihandle*) - element handle
' The default return value for all callbacks is "i" (int). 
' But the return value can be specified using one of the above parameters, 
' after all parameters using "=" to separate it from them.
_DS_ iupClassRegisterCallback(ic as Iclass ptr, name_ as const zstring ptr, format_ as const zstring ptr)

' Returns the format of the parameters of a registered callback. 
' If NULL then the default callback definition is assumed.
_DF_ iupClassCallbackGetFormat(ic as Iclass ptr, name_ as const zstring ptr) as zstring ptr

' Class Object Functions
' Stubs for the class methods. They implement inheritance and check if method is NULL.

' Calls Iclass::Create method. 
_DF_ iupClassObjectCreate(ih as IHandle ptr, params as any ptr ptr) as integer

' Calls Iclass::Map method. 
_DF_ iupClassObjectMap(ih as IHandle ptr) as integer

' Calls Iclass::UnMap method. 
_DS_ iupClassObjectUnMap(ih as IHandle ptr)

' Calls Iclass::Destroy method. 
_DS_ iupClassObjectDestroy(ih as IHandle ptr)

' Calls Iclass::GetInnerContainer method.
' The parent class is ignored. If necessary the child class must handle the parent class internally.
_DF_ iupClassObjectGetInnerContainer(ih as IHandle ptr) as Ihandle ptr

' Calls Iclass::GetInnerNativeContainerHandle method. Returns ih->handle if there is no inner parent.
' The parent class is ignored. If necessary the child class must handle the parent class internally.
_DF_ iupClassObjectGetInnerNativeContainerHandle(ih as IHandle ptr, child as Ihandle ptr) as any ptr

' Calls Iclass::ChildAdded method. 
_DS_ iupClassObjectChildAdded(ih as IHandle ptr, child as Ihandle ptr)

' Calls Iclass::ChildRemoved method. 
_DS_ iupClassObjectChildRemoved(ih as IHandle ptr, child as Ihandle ptr)

' Calls Iclass::LayoutUpdate method. 
_DS_ iupClassObjectLayoutUpdate(ih as IHandle ptr)

' Calls Iclass::ComputeNaturalSize method. 
_DS_ iupClassObjectComputeNaturalSize(ih as IHandle ptr, w as integer ptr, h as integer ptr, children_expand as integer ptr)

' Calls Iclass::SetChildrenCurrentSize method. 
_DS_ iupClassObjectSetChildrenCurrentSize(ih as IHandle ptr, shrink as integer)

' Calls Iclass::SetChildrenPosition method. 
_DS_ iupClassObjectSetChildrenPosition(ih as IHandle ptr, x as integer, y as integer)

' Calls Iclass::DlgPopup method. 
_DF_ iupClassObjectDlgPopup(ih as IHandle ptr, x as integer, y as integer) as integer

' Handle attributes, but since the attribute function table is shared by the class hierarchy,
' the attribute function is retrieved only from the current class.
' Set is called from iupAttribUpdate (IupMap), IupStoreAttribute and IupSetAttribute.
' Get is called only from IupGetAttribute.
_DF_ iupClassObjectSetAttribute(ih as IHandle ptr, name_ as const zstring ptr, valueas as const zstring ptr, inherit as integer ptr) as integer
_DF_ iupClassObjectGetAttribute(ih as IHandle ptr, name_ as const zstring ptr, def_valueas as const zstring ptr ptr, inherit as integer ptr) as zstring ptr
_DF_ iupClassObjectSetAttributeId(ih as IHandle ptr, name_ as const zstring ptr, id as integer, value as const zstring ptr) as integer
_DF_ iupClassObjectGetAttributeId(ih as IHandle ptr, name_ as const zstring ptr, id as integer) as zstring ptr
_DF_ iupClassObjectSetAttributeId2(ih as IHandle ptr, name_ as const zstring ptr, id1 as integer, id2 as integer, value as const zstring ptr) as integer
_DF_ iupClassObjectGetAttributeId2(ih as IHandle ptr, name_ as const zstring ptr, id1 as integer, id2 as integer) as zstring ptr

' Used only in iupAttribGetStr
_DS_ iupClassObjectGetAttributeInfo(ih as IHandle ptr, name_ as const zstring ptr, def_value as const zstring ptr ptr, inherit as integer ptr)

' Used only in iupAttribIsPointer
_DF_ iupClassObjectAttribIsNotString(ih as IHandle ptr, name_ as const zstring ptr) as integer

' Used only in iupAttribUpdateFromParent
_DF_ iupClassObjectCurAttribIsInherit(ic as Iclass ptr) as integer

' Used in iupObjectCreate and IupMap
_DS_ iupClassObjectEnsureDefaultAttributes(ih as IHandle ptr)

' Used in IupLayoutDialog
_DF_ iupClassAttribIsRegistered(ic as Iclass ptr, name_ as const zstring ptr) as integer
_DS_ iupClassGetAttribNameInfo(ic as Iclass ptr, name_ as const zstring ptr, def_value as const zstring ptr ptr, flags as integer ptr)

end extern

/' Other functions declared in iup.bi and implemented here. 
IupGetClassType
IupGetClassName
IupClassMatch
'/


#endif
